from lailib.image.augment import *
import pytest


class TestJPGCompress:
    def test_acc_mc(self):
        img_rgb = np.array(np.random.rand(802,604,3)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_m(self):
        img_rgb = np.array(np.random.rand(128,256)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_xlc(self):
        img_rgb = np.array(np.random.rand(4000,4000,3)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_xsc(self):
        img_rgb = np.array(np.random.rand(2,2,3)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_xl(self):
        img_rgb = np.array(np.random.rand(4000,4000)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_xs(self):
        img_rgb = np.array(np.random.rand(2,2)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb)
        assert img_re.shape == img_rgb.shape

    def test_acc_low_qual(self):
        img_rgb = np.array(np.random.rand(800,458,3)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb,0)
        assert img_re.shape == img_rgb.shape

    def test_acc_high_qual(self):
        img_rgb = np.array(np.random.rand(128,256,3)*255,dtype=np.uint8)
        img_re = jpg_compress(img_rgb,100)
        assert img_re.shape == img_rgb.shape

    def test_float_c(self):
        img_rgb = np.random.rand(128,256,3)
        with pytest.raises(ValueError):
            img_re = jpg_compress(img_rgb)

    def test_float(self):
        img_rgb = np.random.rand(128,256)
        with pytest.raises(ValueError):
            img_re = jpg_compress(img_rgb)



from lailib.image.convert import *
import numpy as np
import os
import cv2


class TestBase64:
    def test_acc_base64_color(self, tmpdir):
        img_rgb = np.array(np.random.rand(128,256,3)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img_rgb)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str)
        assert img_from_str.shape == img_rgb.shape
        assert np.array_equal(img_from_str,img_rgb)


    def test_acc_base64_gray(self, tmpdir):
        img = np.array(np.random.rand(128,256)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str,rgb=False)
        assert img_from_str.shape == img.shape
        assert np.array_equal(img_from_str,img)

    def test_acc_base64_color_xl(self, tmpdir):
        img_rgb = np.array(np.random.rand(4000,4000,3)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img_rgb)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str)
        assert img_from_str.shape == img_rgb.shape
        assert np.array_equal(img_from_str,img_rgb)


    def test_acc_base64_gray_xl(self, tmpdir):
        img = np.array(np.random.rand(4000,4000)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str,rgb=False)
        assert img_from_str.shape == img.shape
        assert np.array_equal(img_from_str,img)

    def test_acc_base64_color_xs(self, tmpdir):
        img_rgb = np.array(np.random.rand(2,2,3)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img_rgb)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str)
        assert img_from_str.shape == img_rgb.shape
        assert np.array_equal(img_from_str,img_rgb)


    def test_acc_base64_gray_xs(self, tmpdir):
        img = np.array(np.random.rand(2,2)*255,dtype=np.uint8)
        img_path = os.path.join(tmpdir,"test_color.png")
        cv2.imwrite(img_path,img)
        base64_str = base64_from_file(img_path)
        img_from_str = base64_to_img(base64_str,rgb=False)
        assert img_from_str.shape == img.shape
        assert np.array_equal(img_from_str,img)


import numpy as np
import cv2

def otsu_thresh(im):
    '''
    binarize image with otsu algorithm
    :param im(ndarray): uint8 numpy array
    :return: binarized image as uint8 numpy array
    '''
    _, binarized = cv2.threshold(im, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
    return binarized

def vanilla_thresh(im):
    '''
    binarize image with a fixed threshold (170)
    :param im(ndarray): uint8 numpy array
    :return: binarized image as uint8 numpy array
    '''
    _, binarized = cv2.threshold(im, 0, 170, cv2.THRESH_BINARY)
    return binarized



import numpy as np
from lailib.image.binarize import otsu_thresh

def crop_boundary_and_padding(im, padding=0, binarized=None):
    '''
    crop and padding objects and text imgs with pure black border, then padding.
    ex:
    for input
        0   255 0   0
        0   255 255 0
        0   255 0   0
        0   0   0   0
    output is
        255 0
        255 255
        255 0
    :param image_cv(ndarray): input image, uint8 numpy array, assumed to be gray scale
    :param padding(int or list of ints): if padding is a scalar, padding will be applied to left, right, upside and downside of an image.
                    if padding is a list of size 4, each entry in the list corresponds to left, right, upside and
                    downside of an image.
    :return: cropped image (uint8)
    '''
    if len(im.shape) != 2 or im.dtype != np.uint8:
        raise TypeError('input image must be gray scale image as uint8 ndarray')
    if not binarized is None:
        if binarized.shape != im.shape:
            raise ValueError('binarized image shape {} must meet in image shape {}'.format(binarized.shape, im.shape))
        if binarized.dtype != np.uint8:
            raise TypeError('binarized mask must be uint8 ndarray')
    else:
        binarized = otsu_thresh(im)
    if np.sum(binarized) == 0:
        raise ValueError('In image for crop function is all zero')

    if isinstance(padding, int):
        padding = [padding] * 4
    elif not (isinstance(padding, list) and len(padding) == 4):
        raise TypeError('padding in crop function must be int or list of size 4')

    # use binarized image to get vertical and horizontal boundaries
    col_sums = np.sum(binarized, axis=1)
    row_start = np.where(col_sums)[0][0]
    rev_col_sum = np.flip(col_sums, axis=0)
    rev_end = np.where(rev_col_sum)[0][0]
    row_end = col_sums.shape[0] - rev_end

    row_sums = np.sum(binarized, axis=0)
    col_start = np.where(row_sums)[0][0]
    rev_row_sum = np.flip(row_sums, axis=0)
    rev_end = np.where(rev_row_sum)[0][0]
    col_end = row_sums.shape[0] - rev_end

    cropped = im[row_start:row_end, col_start:col_end]

    cropped_height, cropped_width = cropped.shape
    # TODO add rand noise to height
    final_out = np.zeros((cropped_height + padding[2] + padding[3],
                          cropped_width + padding[0] + padding[1]), dtype = np.uint8)

    out_height, out_width = final_out.shape
    # do not directly use -padding as the value for end side because it can't deal with padding = 0
    final_out[padding[2]: out_height - padding[3], padding[0]: out_width - padding[1]] = cropped
    return final_out





import base64
import cv2
import numpy as np


def base64_to_img(str,rgb = True):
    """
    convert base64 string to numpy image
    :param str: (string)base64 encoded string of image
    :param rgb: (Bool)if decoded image is rgb,
    :return:
    """
    mode = 0
    if rgb:
        mode = 1
    img = base64.b64decode(str)
    img_np = np.fromstring(img, np.uint8)
    img = cv2.imdecode(img_np,mode)
    return img


def base64_from_file(path):
    """
    get base64 image string from file path
    :param path: (string)path to image file
    :return: (string)base64 image string
    """
    with open(path, 'rb') as handler:
        return base64.b64encode(handler.read()).decode('utf-8')